package de.hft.stuttgart.citydoctor2.ziploader;

import de.hft.stuttgart.citydoctor2.check.Checker;
import de.hft.stuttgart.citydoctor2.datastructure.CityDoctorModel;
import de.hft.stuttgart.citydoctor2.parser.CityGmlParseException;
import de.hft.stuttgart.citydoctor2.parser.CityGmlParser;
import de.hft.stuttgart.citydoctor2.parser.InvalidGmlFileException;
import de.hft.stuttgart.citydoctor2.parser.ParserConfiguration;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.io.IOException;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.util.zip.ZipInputStream;

public class CityGmlZipEntry {

    private static final Logger logger = LogManager.getLogger(CityGmlZipEntry.class);

    private String fileName;
    private String archivePath;
    private CityDoctorModel model;

    private boolean erroneousEntry = false;
    private boolean isLibraryObject = false;
    private boolean validated = false;
    private ZipEntryErrorType errorType = null;

    public static CityGmlZipEntry of(ZipEntry entry, ParserConfiguration config, String archivePath, Path tempDir, InputStream is){
        CityGmlZipEntry cgzEntry = new CityGmlZipEntry();
        cgzEntry.fileName = entry.getName();
        cgzEntry.archivePath = archivePath;
        if ((double) entry.getSize() /1024/1024 > 20) {
            return new ErroneousEntry(entry, ZipEntryErrorType.EXCESSIVE_FILESIZE);
        } else {
            try {
                Files.copy(is, Path.of(tempDir.toString(), entry.getName()));
                CityGmlParser.gagLogger(true);
                cgzEntry.model = CityGmlParser.parseCityGmlFile(tempDir.toString() + File.separator + entry.getName(), config);
            } catch (CityGmlParseException | InvalidGmlFileException e) {
                logger.error(e);
                return new ErroneousEntry(entry, ZipEntryErrorType.INVALID_CITY_GML_FILE);
            } catch (IOException e){
                logger.error(e);
                return new ErroneousEntry(entry, ZipEntryErrorType.IO_ERROR);
            }
        }
        return cgzEntry;
    }

    private CityGmlZipEntry(String filename, CityDoctorModel model){
        this.fileName = filename;
        this.model = model;
    }

    protected CityGmlZipEntry(ZipEntry entry) {
        this.fileName = entry.getName();
        this.model = null;
    }
    public void validateModel(){
        Checker checker = new Checker(this.getModel());
        checker.runChecks();
        validated = true;
    }

    public boolean isValidated(){
        return validated;
    }

    public String getFileName() {
        return fileName;
    }

    public CityDoctorModel getModel() {
        return model;
    }

}
